//! micro:bit buttons to switch patterns.

use embassy_executor::{SpawnError, Spawner};
use embassy_nrf::gpio::{AnyPin, Input, Pull};

use crate::state;

/// Wait for a button press on the provided pin and then publish the provided message.
#[embassy_executor::task(pool_size = 2)]
async fn pattern_switch(button: AnyPin, pattern_move: state::StateChange) {
    let mut button = Input::new(button, Pull::Up);
    let state_pub = state::STATE.publisher().unwrap();

    loop {
        button.wait_for_low().await;
        button.wait_for_high().await;
        state_pub.publish_immediate(pattern_move);
    }
}

/// Spawn tasks for the left and right button.
/// The left button sends a PatternLeft message, the right button a PatternRight.
pub fn spawn(
    spawner: Spawner,
    button_left: AnyPin,
    button_right: AnyPin,
) -> Result<(), SpawnError> {
    spawner.spawn(pattern_switch(button_left, state::StateChange::PatternLeft))?;
    spawner.spawn(pattern_switch(
        button_right,
        state::StateChange::PatternRight,
    ))?;

    Ok(())
}
